    public class TestLevelDict<string, int, TValueStorage> : Dictionary<string, int>, ISerializationCallbackReceiver
    {
        [SerializeField]
        string[] m_keys;
        [SerializeField]
        TValueStorage[] m_values;

        public TestLevelDict()
        {
        }

        public TestLevelDict(IDictionary<string, int> dict) : base(dict.Count)
        {
            foreach (var kvp in dict)
            {
                this[kvp.Key] = kvp.Value;
            }
        }

        protected TestLevelDict(SerializationInfo info, StreamingContext context) : base(info, context) { }

        protected abstract void SetValue(TValueStorage[] storage, int i, int value);
        protected abstract int GetValue(TValueStorage[] storage, int i);

        public void CopyFrom(IDictionary<string, int> dict)
        {
            this.Clear();
            foreach (var kvp in dict)
            {
                this[kvp.Key] = kvp.Value;
            }
        }

        public void OnAfterDeserialize()
        {
            if (m_keys != null && m_values != null && m_keys.Length == m_values.Length)
            {
                this.Clear();
                int n = m_keys.Length;
                for (int i = 0; i < n; ++i)
                {
                    this[m_keys[i]] = GetValue(m_values, i);
                }

                m_keys = null;
                m_values = null;
            }

        }

        public void OnBeforeSerialize()
        {
            int n = this.Count;
            m_keys = new TKey[n];
            m_values = new TValueStorage[n];

            int i = 0;
            foreach (var kvp in this)
            {
                m_keys[i] = kvp.Key;
                SetValue(m_values, i, kvp.Value);
                ++i;
            }
        }
    }

    public class SerializableDictionary<string, int> : SerializableDictionaryBase<string, int, int>
    {
        public SerializableDictionary()
        {
        }

        public SerializableDictionary(IDictionary<string, int> dict) : base(dict)
        {
        }

        protected SerializableDictionary(SerializationInfo info, StreamingContext context) : base(info, context) { }

        protected override TValue GetValue(int[] storage, int i)
        {
            return storage[i];
        }

        protected override void SetValue(int[] storage, int i, int value)
        {
            storage[i] = value;
        }
    }

    public static class SerializableDictionary
    {
        public class Storage<T>
        {
            public T data;
        }
    }

    public class SerializableDictionary<string, int, TValueStorage> : SerializableDictionaryBase<string, int, TValueStorage> where TValueStorage : SerializableDictionary.Storage<int>, new()
    {
        public SerializableDictionary()
        {
        }

        public SerializableDictionary(IDictionary<string, int> dict) : base(dict)
        {
        }

        protected SerializableDictionary(SerializationInfo info, StreamingContext context) : base(info, context) { }

        protected override int GetValue(TValueStorage[] storage, int i)
        {
            return storage[i].data;
        }

        protected override void SetValue(TValueStorage[] storage, int i, int value)
        {
            storage[i] = new TValueStorage();
            storage[i].data = value;
        }